using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.IO;
using System.Runtime.InteropServices;
using System.Runtime.InteropServices.ComTypes;
using System.Windows.Forms;
using GCFilePreviewer;



namespace Thumbnailer
{
    [ComVisible(true), ClassInterface(ClassInterfaceType.None)]
    [ProgId("GibbsCAM.ThumbnailProvider"), Guid("200F58FD-529E-4eca-A747-67F5C64801A2")]
    public class ThumbnailProvider : IThumbnailProvider, IInitializeWithStream
    {
        public static void Log(String text)
        {
            if (false)
            {
                StreamWriter w = File.AppendText("C:\\log\\log.txt");
                w.WriteLine(":" + DateTime.Now.ToString() + " {0}", text);
                // Update the underlying file.
                w.Flush();
                w.Close();
            }
        }

        #region IInitializeWithStream

        protected IStream BaseStream
        {
            get { return (_stream); }
            set { this._stream = value; }
        }

        public void Initialize(IStream stream, int grfMode)
        {
            //Log("Initializing Stream");
            this.BaseStream = stream;
        }

        #endregion

        #region IThumbnailProvider

        public void GetThumbnail(int cx, out IntPtr hBitmap, out WTS_ALPHATYPE bitmapType)
        {

            hBitmap = IntPtr.Zero;
            bitmapType = WTS_ALPHATYPE.WTSAT_RGB;
            try
            {
                /*using (Bitmap Thumbnail = new Bitmap(cx, cx))
                {
                    GCImage gcimg = new GCImage();
                    byte[] file = GetStreamContents();
                    MemoryStream stream = new MemoryStream(file);
                    Log("Loading Image From stream");
                    int size = gcimg.Load(stream);
                    if( size > 0 )
                    {
                        sbyte[] temp = gcimg.GetBuffer();
                        GCHandle pinnedArray = GCHandle.Alloc(temp, GCHandleType.Pinned);
                        IntPtr pointer = pinnedArray.AddrOfPinnedObject();
                        Bitmap new_bitmap = new Bitmap(188, 141, 564, System.Drawing.Imaging.PixelFormat.Format24bppRgb, pointer);
                        hBitmap = new_bitmap.GetHbitmap();
                        Log("Done");
                    }
                    else
                        Log("Done no image to extract");
                }*/
            }
            catch { } // A dirty cop-out.

        }

        protected IStream Stream
        {
            get { return (_stream); }
            set { this._stream = value; }
        }

        /// <summary>
        /// The ThumbnailProvider returns a stream that we convert to a byte array.
        /// </summary>
        /// <returns></returns>
        protected byte[] GetStreamContents()
        {
            System.Diagnostics.Debug.Assert(this._stream != null);

            // How big do we think this stream is?
            System.Runtime.InteropServices.ComTypes.STATSTG statData;
            _stream.Stat(out statData, 1); // 1 means don't bother with the name.

            // Allocate some memory.
            byte[] bytes = new byte[statData.cbSize];

            IntPtr p = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(UInt64)));

            this._stream.Read(bytes, bytes.Length, p);

            Marshal.FreeCoTaskMem(p);

            return (bytes);
        }

        private IStream _stream;

        #endregion
    }
}



namespace Thumbnailer
{

    /// <summary>
    /// Defines the format of a bitmap returned by an <see cref="IThumbnailProvider"/>.
    /// </summary>
    public enum WTS_ALPHATYPE
    {
        /// <summary>
        /// The bitmap is an unknown format. The Shell tries nonetheless to detect whether the image has an alpha channel.
        /// </summary>
        WTSAT_UNKNOWN = 0,
        /// <summary>
        /// The bitmap is an RGB image without alpha. The alpha channel is invalid and the Shell ignores it.
        /// </summary>
        WTSAT_RGB = 1,
        /// <summary>
        /// The bitmap is an ARGB image with a valid alpha channel.
        /// </summary>
        WTSAT_ARGB = 2,
    }

    /// <summary>
    /// Exposes a method for getting a thumbnail image.
    /// </summary>
    [ComVisible(true), Guid("e357fccd-a995-4576-b01f-234630154e96"), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    public interface IThumbnailProvider
    {
        /// <summary>
        /// Retrieves a thumbnail image and alpha type. 
        /// </summary>
        /// <param name="cx">The maximum thumbnail size, in pixels. The Shell draws the returned bitmap at this size or smaller. The returned bitmap should fit into a square of width and height <paramref name="cx"/>, though it does not need to be a square image. The Shell scales the bitmap to render at lower sizes. For example, if the image has a 6:4 aspect ratio, then the returned bitmap should also have a 6:4 aspect ratio.</param>
        /// <param name="hBitmap">When this method returns, contains a pointer to the thumbnail image handle. The image must be a device-independent bitmap (DIB) section and 32 bits per pixel. The Shell scales down the bitmap if its width or height is larger than the size specified by cx. The Shell always respects the aspect ratio and never scales a bitmap larger than its original size.</param>
        /// <param name="bitmapType">Specifies the format of the output bitmap.</param>
        void GetThumbnail(int cx, out IntPtr hBitmap, out WTS_ALPHATYPE bitmapType);
    }

    /// <summary>
    /// Provides a method used to initialize a handler, such as a property handler, thumbnail provider, or preview handler, with a file stream.
    /// </summary>
    [ComVisible(true), Guid("b824b49d-22ac-4161-ac8a-9916e8fa3f7f"), InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    public interface IInitializeWithStream
    {
        /// <summary>
        /// Initializes a handler with a file stream.
        /// </summary>
        /// <param name="stream">Pointer to an <see cref="IStream"/> interface that represents the file stream source.</param>
        /// <param name="grfMode">Indicates the access mode for <paramref name="stream"/>.</param>
        void Initialize(IStream stream, int grfMode);
    }

}
